﻿using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using KSP.UI.Screens;
using System.Reflection;
using System.Diagnostics;



namespace KSPBugDemo
{
    public delegate bool TryParse<T>(string str, out T value);


    [KSPAddon(KSPAddon.Startup.SpaceCentre, true)] // Determines when plugin starts.
    public class KSPBugDemo : MonoBehaviour
    {
       
        private static ApplicationLauncherButton _appLauncherButton;
        static List<ApplicationLauncherButton> appListModHidden;

        const int WINDOW_WIDTH = 400;
        const int WINDOW_HEIGHT = 200;
        private Rect windowRect;

        private void Start()
        {
            AddAppLauncher();
            windowRect = new Rect(Screen.width / 2 - WINDOW_WIDTH / 2, Screen.height/2 - WINDOW_HEIGHT / 2, WINDOW_WIDTH, WINDOW_HEIGHT);

            // following needed to fix a stock bug
            appListModHidden = (List<ApplicationLauncherButton>)typeof(ApplicationLauncher).GetField("appListModHidden", BindingFlags.NonPublic | BindingFlags.Instance).GetValue(ApplicationLauncher.Instance);
            DontDestroyOnLoad(this);
        }

#if true
        double lasttimecheck = 0;
        GameScenes lastScene = GameScenes.LOADING;
        double lastTime = 0;

        // Enable the following section to activate the workaround for the KSP toolbar bug

        /// <summary>
        /// Checks after every scene change to see if the button is hidden,
        /// if it is hidden, then disable it.
        /// </summary>
        private void FixedUpdate()
        {
            if (HighLogic.LoadedScene != lastScene)
            {
                lastScene = HighLogic.LoadedScene;
                lastTime = Time.fixedTime;
            }
            if (Time.fixedTime - lastTime < 2)
            {
                if (Time.fixedTime - lasttimecheck > .1)
                {
                    lasttimecheck = Time.fixedTime;
                    // following fixes a stock bug
                    if (appListModHidden.Contains(global::KSPBugDemo.KSPBugDemo.appButton))
                    {
                        global::KSPBugDemo.KSPBugDemo.appButton.gameObject.SetActive(false);
                        if (global::KSPBugDemo.KSPBugDemo.appButton.enabled)
                            global::KSPBugDemo.KSPBugDemo.appButton.onDisable();

                    }
                }
                
            }
        }
#endif

        public static ApplicationLauncherButton appButton
        {
            get { return _appLauncherButton; }
        }



        public void Awake() // Called after scene (designated w/ KSPAddon) loads, but before Start().  Init data here.
        {
            GameEvents.onGUIApplicationLauncherReady.Add(AddAppLauncher);
            GameEvents.onGUIApplicationLauncherDestroyed.Add(RemoveAppLauncher);
           
        }
        const string TexturePath = "KSPBugDemo/";

        private void AddAppLauncher()
        {
            if (_appLauncherButton != null)
            {
                UnityEngine.Debug.Log(
                    "Not adding to ApplicationLauncher, button already exists (yet onGUIApplicationLauncherReady was called?)");
                return;
            }
            var applauncher = ApplicationLauncher.Instance;
            if (applauncher == null)
            {
                UnityEngine.Debug.Log("Cannot add to ApplicationLauncher, instance was null");
                return;
            }
            const ApplicationLauncher.AppScenes scenes =
                ApplicationLauncher.AppScenes.FLIGHT |
                ApplicationLauncher.AppScenes.MAPVIEW |
                ApplicationLauncher.AppScenes.TRACKSTATION;

            _appLauncherButton = applauncher.AddModApplication(
                ToggleWindow, // onTrue
                ToggleWindow, // onFalse
                () => { }, // onHover
                () => { }, // onHoverOut
                () => { }, // onEnable
                () => { }, // onDisable
                scenes, // visibleInScenes
                 GameDatabase.Instance.GetTexture(TexturePath + "spacetux", false)  // texture
            );
        }

        bool visible = false;
        void ToggleWindow()
        {
            visible = !visible;
        }

         
        void OnGUI()
        {

            if (!visible)
                return;
                
            
            GUI.skin = HighLogic.Skin;
            windowRect = GUILayout.Window(44444, windowRect, Window, "KSP Bug Demo Window");

        }

        public void Window(int windowID)
        {
            if (!enabled)
                return;
            GUI.skin = HighLogic.Skin;

            GUILayout.BeginHorizontal();
            GUILayout.EndHorizontal();
            GUILayout.BeginHorizontal();
            GUILayout.FlexibleSpace();
            GUILayout.Label("Test Window");
            GUILayout.FlexibleSpace();
            GUILayout.EndHorizontal();
            GUI.DragWindow();
        }

        private void RemoveAppLauncher()
        {
            var applauncher = ApplicationLauncher.Instance;
            if (applauncher == null)
            {
                UnityEngine.Debug.Log("Cannot remove from ApplicationLauncher, instance was null");
                return;
            }
            if (_appLauncherButton == null)
                return;
            applauncher.RemoveModApplication(_appLauncherButton);
            _appLauncherButton = null;
        }

    // End of class.
    }

 
}
